@@ -10,12 +10,14 @@ use oura::{
         BootstrapResult, FilterProvider, PartialBootstrapResult, SinkProvider, SourceProvider,
         StageReceiver,
     },
-    utils::{cursor, ChainWellKnownInfo, Utils, WithUtils},
+    sources::MagicArg,
+    utils::{cursor, metrics, ChainWellKnownInfo, Utils, WithUtils},
     Error,
 };
 
 use oura::filters::noop::Config as NoopFilterConfig;
 use oura::filters::selection::Config as SelectionConfig;
+use oura::sinks::assert::Config as AssertConfig;
 use oura::sinks::stdout::Config as StdoutConfig;
 use oura::sinks::terminal::Config as TerminalConfig;
 use oura::sources::n2c::Config as N2CConfig;
@@ -50,6 +52,13 @@ fn bootstrap_source(config: Source, utils: Arc<Utils>) -> PartialBootstrapResult
     }
 }
 
+fn infer_magic_from_source(config: &Source) -> Option<MagicArg> {
+    match config {
+        Source::N2C(config) => config.magic.clone(),
+        Source::N2N(config) => config.magic.clone(),
+    }
+}
+
 #[derive(Debug, Deserialize)]
 #[serde(tag = "type")]
 enum Filter {
@@ -77,6 +86,7 @@ impl FilterProvider for Filter {
 enum Sink {
     Terminal(TerminalConfig),
     Stdout(StdoutConfig),
+    Assert(AssertConfig),
 
     #[cfg(feature = "logs")]
     Logs(WriterConfig),
@@ -95,6 +105,7 @@ fn bootstrap_sink(config: Sink, input: StageReceiver, utils: Arc<Utils>) -> Boot
     match config {
         Sink::Terminal(c) => WithUtils::new(c, utils).bootstrap(input),
         Sink::Stdout(c) => WithUtils::new(c, utils).bootstrap(input),
+        Sink::Assert(c) => WithUtils::new(c, utils).bootstrap(input),
 
         #[cfg(feature = "logs")]
         Sink::Logs(c) => WithUtils::new(c, utils).bootstrap(input),
@@ -122,36 +133,58 @@ struct ConfigRoot {
     chain: Option<ChainWellKnownInfo>,
 
     cursor: Option<cursor::Config>,
+
+    metrics: Option<metrics::Config>,
 }
 
 impl ConfigRoot {
     pub fn new(explicit_file: Option<String>) -> Result<Self, ConfigError> {
-        let mut s = Config::default();
+        let mut s = Config::builder();
 
         // our base config will always be in /etc/oura
-        s.merge(File::with_name("/etc/oura/daemon.toml").required(false))?;
+        s = s.add_source(File::with_name("/etc/oura/daemon.toml").required(false));
 
         // but we can override it by having a file in the working dir
-        s.merge(File::with_name("oura.toml").required(false))?;
+        s = s.add_source(File::with_name("oura.toml").required(false));
 
         // if an explicit file was passed, then we load it as mandatory
         if let Some(explicit) = explicit_file {
-            s.merge(File::with_name(&explicit).required(true))?;
+            s = s.add_source(File::with_name(&explicit).required(true));
         }
 
         // finally, we use env vars to make some last-step overrides
-        s.merge(Environment::with_prefix("OURA").separator("_"))?;
+        s = s.add_source(Environment::with_prefix("OURA").separator("_"));
 
-        s.try_into()
+        s.build()?.try_deserialize()
     }
 }
 
-fn bootstrap_utils(chain: Option<ChainWellKnownInfo>, cursor: Option<cursor::Config>) -> Utils {
-    let well_known = chain.unwrap_or_default();
+fn define_chain_info(
+    explicit: Option<ChainWellKnownInfo>,
+    magic: &MagicArg,
+) -> Result<ChainWellKnownInfo, Error> {
+    match explicit {
+        Some(x) => Ok(x),
+        None => ChainWellKnownInfo::try_from_magic(**magic),
+    }
+}
 
-    let cursor = cursor.map(cursor::Provider::initialize);
+fn bootstrap_utils(
+    chain: ChainWellKnownInfo,
+    cursor: Option<cursor::Config>,
+    metrics: Option<metrics::Config>,
+) -> Utils {
+    let mut utils = Utils::new(chain);
 
-    Utils::new(well_known, cursor)
+    if let Some(cursor) = cursor {
+        utils = utils.with_cursor(cursor);
+    }
+
+    if let Some(metrics) = metrics {
+        utils = utils.with_metrics(metrics);
+    }
+
+    utils
 }
 
 /// Sets up the whole pipeline from configuration
@@ -162,9 +195,14 @@ fn bootstrap(config: ConfigRoot) -> Result<Vec<JoinHandle<()>>, Error> {
         sink,
         chain,
         cursor,
+        metrics,
     } = config;
 
-    let utils = Arc::new(bootstrap_utils(chain, cursor));
+    let magic = infer_magic_from_source(&source).unwrap_or_default();
+
+    let chain = define_chain_info(chain, &magic)?;
+
+    let utils = Arc::new(bootstrap_utils(chain, cursor, metrics));
 
     let mut threads = Vec::with_capacity(10);
 
