@@ -1,6 +1,6 @@
 use serde_json::Value as JsonValue;
 
-use pallas::ledger::alonzo::Metadatum;
+use pallas::ledger::primitives::alonzo::Metadatum;
 
 use crate::{model::CIP25AssetRecord, Error};
 
@@ -75,35 +75,34 @@ impl EventWriter {
         policy: &str,
         content: &Metadatum,
     ) -> Result<(), Error> {
-        let entries = match content {
-            Metadatum::Map(map) => map,
-            _ => return Err("expected 721 policy content to be a map".into()),
-        };
-
-        for (key, sub_content) in entries.iter() {
-            if let Some(asset) = is_asset_key(key) {
-                let record = self.to_cip25_asset_record(version, policy, &asset, sub_content)?;
-                self.append_from(record)?;
+        if let Metadatum::Map(entries) = content {
+            for (key, sub_content) in entries.iter() {
+                if let Some(asset) = is_asset_key(key) {
+                    let record =
+                        self.to_cip25_asset_record(version, policy, &asset, sub_content)?;
+                    self.append_from(record)?;
+                }
             }
+        } else {
+            log::warn!("invalid metadatum type for policy inside 721 label");
         }
 
         Ok(())
     }
 
     pub(crate) fn crawl_metadata_label_721(&self, content: &Metadatum) -> Result<(), Error> {
-        let entries = match content {
-            Metadatum::Map(map) => map,
-            _ => return Err("expected 721 content to be a map".into()),
-        };
-
         let version = self
             .search_cip25_version(content)
             .unwrap_or_else(|| "1.0".to_string());
 
-        for (key, sub_content) in entries.iter() {
-            if let Some(policy) = is_policy_key(key) {
-                self.crawl_721_policy(&version, &policy, sub_content)?;
+        if let Metadatum::Map(entries) = content {
+            for (key, sub_content) in entries.iter() {
+                if let Some(policy) = is_policy_key(key) {
+                    self.crawl_721_policy(&version, &policy, sub_content)?;
+                }
             }
+        } else {
+            log::warn!("invalid metadatum type for 721 label");
         }
 
         Ok(())
