@@ -2,16 +2,17 @@ use std::collections::HashMap;
 
 use minicbor::bytes::ByteVec;
 use pallas::crypto::hash::Hash;
-use pallas::ledger::alonzo::{
+use pallas::ledger::primitives::alonzo::{
     self as alonzo, AuxiliaryData, Block, Certificate, InstantaneousRewardSource,
     InstantaneousRewardTarget, Metadatum, Relay, TransactionInput, TransactionOutput, Value,
 };
-use pallas::ledger::alonzo::{NetworkId, TransactionBody, TransactionBodyComponent};
+use pallas::ledger::primitives::alonzo::{NetworkId, TransactionBody, TransactionBodyComponent};
 
+use pallas::network::miniprotocols::Point;
 use serde_json::{json, Value as JsonValue};
 
 use crate::model::{
-    BlockRecord, EventData, MetadataRecord, MetadatumRendition, MintRecord, OutputAssetRecord,
+    BlockRecord, Era, EventData, MetadataRecord, MetadatumRendition, MintRecord, OutputAssetRecord,
     StakeCredential, TransactionRecord, TxInputRecord, TxOutputRecord,
 };
 
@@ -348,8 +349,15 @@ impl EventWriter {
         Ok(record)
     }
 
-    pub fn to_block_record(&self, source: &Block, hash: &Hash<32>) -> Result<BlockRecord, Error> {
+    pub fn to_block_record(
+        &self,
+        source: &Block,
+        hash: &Hash<32>,
+        cbor: &[u8],
+        era: Era,
+    ) -> Result<BlockRecord, Error> {
         Ok(BlockRecord {
+            era,
             body_size: source.header.header_body.block_body_size as usize,
             issuer_vkey: source.header.header_body.issuer_vkey.to_hex(),
             tx_count: source.transaction_bodies.len(),
@@ -357,6 +365,25 @@ impl EventWriter {
             number: source.header.header_body.block_number,
             slot: source.header.header_body.slot,
             previous_hash: hex::encode(source.header.header_body.prev_hash),
+            cbor_hex: match self.config.include_block_cbor {
+                true => hex::encode(cbor).into(),
+                false => None,
+            },
         })
     }
+
+    pub(crate) fn append_rollback_event(&self, point: &Point) -> Result<(), Error> {
+        let data = match point {
+            Point::Origin => EventData::RollBack {
+                block_slot: 0,
+                block_hash: "".to_string(),
+            },
+            Point::Specific(slot, hash) => EventData::RollBack {
+                block_slot: *slot,
+                block_hash: hex::encode(&hash),
+            },
+        };
+
+        self.append(data)
+    }
 }
