@@ -2,10 +2,8 @@ use core::fmt;
 use std::{ops::Deref, str::FromStr};
 
 use log::info;
-use pallas::ouroboros::network::{
-    chainsync::TipFinder,
-    handshake::{MAINNET_MAGIC, TESTNET_MAGIC},
-    machines::{primitives::Point, run_agent},
+use pallas::network::{
+    miniprotocols::{chainsync::TipFinder, run_agent, Point, MAINNET_MAGIC, TESTNET_MAGIC},
     multiplexer::Channel,
 };
 use serde::{de::Visitor, Deserializer};
@@ -48,13 +46,7 @@ impl TryInto<Point> for PointArg {
 
     fn try_into(self) -> Result<Point, Self::Error> {
         let hash = hex::decode(&self.1)?;
-        Ok(Point(self.0, hash))
-    }
-}
-
-impl From<Point> for PointArg {
-    fn from(other: Point) -> Self {
-        PointArg(other.0, hex::encode(&other.1))
+        Ok(Point::Specific(self.0, hash))
     }
 }
 
@@ -79,7 +71,7 @@ impl ToString for PointArg {
     }
 }
 
-#[derive(Debug, Deserialize)]
+#[derive(Debug, Deserialize, Clone)]
 pub struct MagicArg(pub u64);
 
 impl Deref for MagicArg {
@@ -149,14 +141,24 @@ where
     deserializer.deserialize_any(MagicArgVisitor)
 }
 
+#[derive(Debug, Deserialize)]
+#[serde(tag = "type", content = "value")]
+pub enum IntersectArg {
+    Tip,
+    Origin,
+    Point(PointArg),
+    Fallbacks(Vec<PointArg>),
+}
+
 pub(crate) fn find_end_of_chain(
     channel: &mut Channel,
     well_known: &ChainWellKnownInfo,
 ) -> Result<Point, crate::Error> {
-    let point = Point(
+    let point = Point::Specific(
         well_known.shelley_known_slot,
         hex::decode(&well_known.shelley_known_hash)?,
     );
+
     let agent = TipFinder::initial(point);
     let agent = run_agent(agent, channel)?;
     info!("chain point query output: {:?}", agent.output);
@@ -168,24 +170,61 @@ pub(crate) fn find_end_of_chain(
 }
 
 pub(crate) fn define_start_point(
-    explicit_arg: &Option<PointArg>,
+    intersect: &Option<IntersectArg>,
+    since: &Option<PointArg>,
     utils: &Utils,
     cs_channel: &mut Channel,
-) -> Result<Point, Error> {
+) -> Result<Option<Vec<Point>>, Error> {
     let cursor = utils.get_cursor_if_any();
 
-    match (cursor, explicit_arg) {
-        (Some(cursor), _) => {
+    match cursor {
+        Some(cursor) => {
             log::info!("found persisted cursor, will use as starting point");
-            cursor.try_into()
-        }
-        (None, Some(arg)) => {
-            log::info!("explicit 'since' argument, will use as starting point");
-            arg.clone().try_into()
-        }
-        _ => {
-            log::info!("no starting point specified, will use tip of chain");
-            find_end_of_chain(cs_channel, &utils.well_known)
+            let points = vec![cursor.try_into()?];
+
+            Ok(Some(points))
         }
+        None => match intersect {
+            Some(IntersectArg::Fallbacks(x)) => {
+                log::info!("found 'fallbacks' intersect argument, will use as starting point");
+                let points: Result<Vec<_>, _> = x.iter().map(|x| x.clone().try_into()).collect();
+
+                Ok(Some(points?))
+            }
+            Some(IntersectArg::Origin) => {
+                log::info!("found 'origin' instersect argument, will use as starting point");
+
+                Ok(None)
+            }
+            Some(IntersectArg::Point(x)) => {
+                log::info!("found 'point' intersect argument, will use as starting point");
+                let points = vec![x.clone().try_into()?];
+
+                Ok(Some(points))
+            }
+            Some(IntersectArg::Tip) => {
+                log::info!("found 'tip' intersect argument, will use as starting point");
+                let tip = find_end_of_chain(cs_channel, &utils.well_known)?;
+                let points = vec![tip];
+
+                Ok(Some(points))
+            }
+            None => match since {
+                Some(x) => {
+                    log::info!("explicit 'since' argument, will use as starting point");
+                    log::warn!("`since` value is deprecated, please use `intersect`");
+                    let points = vec![x.clone().try_into()?];
+
+                    Ok(Some(points))
+                }
+                None => {
+                    log::info!("no starting point specified, will use tip of chain");
+                    let tip = find_end_of_chain(cs_channel, &utils.well_known)?;
+                    let points = vec![tip];
+
+                    Ok(Some(points))
+                }
+            },
+        },
     }
 }
