@@ -6,9 +6,8 @@ use net2::TcpStreamExt;
 
 use log::info;
 
-use pallas::ouroboros::network::{
-    handshake::{n2c, MAINNET_MAGIC},
-    machines::{primitives::Point, run_agent},
+use pallas::network::{
+    miniprotocols::{handshake::n2c, run_agent, MAINNET_MAGIC},
     multiplexer::{Channel, Multiplexer},
 };
 
@@ -19,7 +18,7 @@ use crate::{
     pipelining::{new_inter_stage_channel, PartialBootstrapResult, SourceProvider},
     sources::{
         common::{AddressArg, BearerKind, MagicArg, PointArg},
-        define_start_point,
+        define_start_point, IntersectArg,
     },
     utils::{ChainWellKnownInfo, WithUtils},
     Error,
@@ -34,13 +33,26 @@ pub struct Config {
     #[serde(deserialize_with = "crate::sources::common::deserialize_magic_arg")]
     pub magic: Option<MagicArg>,
 
+    #[deprecated(note = "use intersect value instead")]
     pub since: Option<PointArg>,
 
+    pub intersect: Option<IntersectArg>,
+
     #[deprecated(note = "chain info is now pipeline-wide, use utils")]
     pub well_known: Option<ChainWellKnownInfo>,
 
     #[serde(default)]
     pub mapper: MapperConfig,
+
+    /// Min block depth (# confirmations) required
+    ///
+    /// The min depth a block requires to be considered safe to send down the
+    /// pipeline. This value is used to configure a rollback buffer used
+    /// internally by the stage. A high value (eg: ~6) will reduce the
+    /// probability of seeing rollbacks events. The trade-off is that the stage
+    /// will need some time to fill up the buffer before sending the 1st event.
+    #[serde(default)]
+    pub min_depth: usize,
 }
 
 fn do_handshake(channel: &mut Channel, magic: u64) -> Result<(), Error> {
@@ -91,12 +103,20 @@ impl SourceProvider for WithUtils<Config> {
 
         let mut cs_channel = muxer.use_channel(5);
 
-        let since: Point = define_start_point(&self.inner.since, &self.utils, &mut cs_channel)?;
+        let known_points = define_start_point(
+            &self.inner.intersect,
+            #[allow(deprecated)]
+            &self.inner.since,
+            &self.utils,
+            &mut cs_channel,
+        )?;
 
-        info!("starting from chain point: {:?}", &since);
+        info!("starting chain sync from: {:?}", &known_points);
 
+        let min_depth = self.inner.min_depth;
         let handle = std::thread::spawn(move || {
-            observe_forever(cs_channel, writer, since).expect("chainsync loop failed");
+            observe_forever(cs_channel, writer, known_points, min_depth)
+                .expect("chainsync loop failed");
         });
 
         Ok((handle, output_rx))
